// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: Yes
//
// @file <filename>.H
// File description here
// 
// @brief <very brief class description>
//
// @author F. Malpartida
// ---------------------------------------------------------------------------

// Include header files
// ----------------------------------------------------------------------------
#include "Arduino.h"
#include <inttypes.h>
#include "Wire.h"
#include "thermistor.h"
#include "analogTemp.h"

// Definitions
// ----------------------------------------------------------------------------


// Static  member variable definitions
// ----------------------------------------------------------------------------

// Static file scope variable definitions
// ----------------------------------------------------------------------------

// Private support functions
// ----------------------------------------------------------------------------

// CLASS METHODS
// ----------------------------------------------------------------------------

// Constructors:
// ---------------------------------------------------------------------------
analogTemp::analogTemp ( void )
{
   _address   = 8; 
   _calOffSet = 282;
   _available = false;
}

// Destructor:
// ---------------------------------------------------------------------------
analogTemp::~analogTemp() { };



// Functions: modifiers (set), selectors (get)
// ---------------------------------------------------------------------------
void analogTemp::init ( void )
{
   analogReference ( INTERNAL );
   _available = true;
}

bool analogTemp::isAvailable ( void )
{
   return (_available);
}


int analogTemp::modeSet ( t_mode mode )
{
   int retVal = 1;
   
   // Only change mode if required, but will be used to see if there is a
   // thermistor conected to the particular address
   // -------------------------------------------------------------------
   switch (mode) 
   {
      case NORMAL:
         retVal = 0;
         break;
      case SHUTDOWN:
         retVal = 1;
         break;

      default:
         break;
   }
   return ( retVal );
}


float analogTemp::readTemperature ( void )
{
   int temp;
   
   analogTemp::readRaw ( temp );
   return ( rawToCesius( temp ) );
}

void  analogTemp::setCalOffset ( int calValue )
{
   _calOffSet = calValue;
}

/**
 Write description of function here. 
 The function should follow these comments.
 
 The function arguments listed with "param" will be compared
 to the declaration and verified.
 
 @param[in]     _inArg1 Description of first function argument.
 @param[out]    _outArg2 Description of second function argument.
 @param[in,out] _inoutArg3 Description of third function argument.
 
 @return Description of returned value.
 */


// PRIVATE METHODS
// -----------------------------------------------------------------------------
float analogTemp::rawToCesius ( int temp )
{
   return (float(temp) - analogTemp::_calOffSet);
}

int analogTemp::readRaw ( int &temperature )
{
#if defined (__AVR_ATmega32U4__)
   ADMUX = 0xC7;                          // activate interal temperature sensor, 
                                          // using 2.56V ref. voltage
   ADCSRB |= _BV(MUX5);
   ADCSRA |= _BV(ADSC);                   // start the conversion
   
   while (bit_is_set(ADCSRA, ADSC));      // ADSC is cleared when the conversion 
                                          // finishes
                                          
   // combine bytes & correct for temperature offset (approximate)
   temperature = ( ADCL | (ADCH << 8));  
#else
   ADMUX = 0xC8;                       // activate interal temperature sensor, 
   // using 1.1V ref. voltage
   ADCSRA |= _BV(ADSC);                // start the conversion
   while (bit_is_set(ADCSRA, ADSC));   // ADSC is cleared when the conversion 
   // finishes
   temperature = (ADCL | (ADCH << 8));
#endif
}

