// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: Yes
//
// @file analogTemp.H
// Class definition for analog temperature thermistor.
// 
// @brief Class implementation to read the internal temperature sensor
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _ANALOG_TEMP_H_
#define _ANALOG_TEMP_H_

// Include header files
// ---------------------------------------------------------------------------
#include "Arduino.h"
#include <inttypes.h>
#include "thermistor.h"


// declarations - constants, variables and types
// ---------------------------------------------------------------------------

// CLASS
// ---------------------------------------------------------------------------
class analogTemp : public thermistor
{
public:   
   // Constructors:
   analogTemp ( void );
   
   // Destructor:
   ~analogTemp();
   
   // Functions: modifiers (set), selectors (get)
   void  init ( void );             // From virtual parent class
   int   modeSet ( t_mode );        // From virtual parent class
   float readTemperature ( void );  // From virtual parent class
   bool  isAvailable ( void );      // From virtual parent class

   void  setCalOffset ( int );
   
   // itterators:
   // Attributes visible by scope of instantiation and use

protected:
   // Attributes visible to descendents
   
private:
   // Local attributes
   uint8_t  _address;    // Analog pin associatated to temp sensor: channel 8
   boolean  _available;  // Indicates if the thermistor at the address is available
   uint16_t _calOffSet;  // Calibration offset
   
   // Private class methods
   int   readRaw ( int & );
   float rawToCesius ( int );
};

// inline function definitions
// ---------------------------------------------------------------------------
#endif  // _ANALOG_TEMP_H_




