#include <LiquidCrystal.h>

// ---------------------------------------------------------------------------
// Created by Francisco Malpartida on 16/04/11.
// Copyright 2011 - Under creative commons license:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: Yes
//
// @file thermalMonitor.c
// File description here
// 
// @brief <very brief class description>
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include <Arduino.h>
#include <Wire.h>

#include "lm75.h"
#include "thermistor.h"

const double FILTER_ALP    = 0.05;
const int    MEAS_PERIOD   = 2;      // In seconds = MEAS_PERIOD * 0.1
const int    REPORT_PERIOD = 20;     // In seconds = REPORT_PERIOD * 0.1 * MEAS_PERIOD
const int    ACTIVITY_PIN  = 13;      // Activity pin LED
const int    CONTRAST_PIN  = 9;
const int    BACKLIGHT_PIN = 7;
const int    CONTRAST      = 80;
const char   *VERSION      = "1.0";

// Available thermistor list in the system
// ---------------------------------------
thermistor *sensor = NULL;
lm75       mySensor( 0b1001000 );
static double filtTemp = 0; 

static void displayTemperature ( const double inst, const double avr )
{
  static double minT = 100.0;
  static double maxT = -30.0;
  
  // Set mins and max temperature readings
  // -------------------------------------
  if ( inst > maxT )
  {
    maxT = inst;
  }
  if ( inst < minT )
  {
    minT = inst;
  }
  
  // Print the information on screen
  Serial.print ("Temp: ");
  Serial.print (inst);
  Serial.println (" C");
  Serial.print (avr);
  Serial.print (" ");
  Serial.print (minT);
  Serial.print (" ");
  Serial.println (maxT);
}


// SETUP - system initialization code 
// -----------------------------------------------------------------------------
void setup ()
{
   
   Wire.begin ();
   Serial.begin ( 57600 );
 
   pinMode(ACTIVITY_PIN, OUTPUT);
  
   // Initialise the temperature sensor
   mySensor.init();
   if ( mySensor.isAvailable() == true )
   {
     sensor = &mySensor;
     filtTemp = sensor->readTemperature();  // Initialise the filter to the first
                                                // reading
   }
}

// MAIN SYSTEM LOOP
// Main system loop, read the temperature, compute PID control, set outputs
// as indicated by control algorithm
// -----------------------------------------------------------------------------
void loop ( void )
{ 
  static int wakeupCounter = MEAS_PERIOD;    // measurement period
  static int reportCounter = REPORT_PERIOD;  // logging period

  wakeupCounter++;
  reportCounter++;
  
    if (sensor != NULL )
    {
      digitalWrite ( ACTIVITY_PIN, HIGH );      
 
      if (wakeupCounter >= MEAS_PERIOD )
      {
        double currentTemp;

        wakeupCounter = 0;
        currentTemp = sensor->readTemperature();
        filtTemp = ( FILTER_ALP * currentTemp) + (( 1.0 - FILTER_ALP ) * filtTemp);
      
        if ( reportCounter >= REPORT_PERIOD )
        {
           displayTemperature ( currentTemp, filtTemp );
           reportCounter = 0;
        }
      }
      delay(20);
      digitalWrite ( ACTIVITY_PIN, LOW );
    }
    
    delay(100);
}

